# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import matplotlib.pyplot as plt
from mpl_toolkits.axes_grid1 import make_axes_locatable
import matplotlib.ticker as mticker
import seaborn as sns
from statannot import add_stat_annotation

import warnings

warnings.filterwarnings("ignore", module="seaborn")

from ..functions import *

grid_hex = change_hex_alpha(gray_hex, 0.25)

bb_lim = (-180, 180)
bb_ticks = [-90, 0, 90]
sc_lim = (0, 360)
sc_ticks = [120, 240]

bb_grid_lst = [
    [[-180, 0], [50, 50], [180, 180]],
    [[-180, 0], [-180, -180], [-100, -100]],
    [[-180, 0], [-100, -100], [50, 50]],
    [[0, 180], [-180, -180], [-50, -50]],
    [[0, 180], [100, 100], [180, 180]],
    [[0, 180], [-50, -50], [100, 100]],
]

sc_x_lst = [[[0, 120]], [[120, 240]], [[240, 360]]]

sc_y_lst = [[[0, 0], [120, 120]], [[120, 120], [240, 240]], [[240, 240], [360, 360]]]

sc_grid_lst = list()
for sc_x in sc_x_lst:
    for sc_y in sc_y_lst:
        sc_grid_lst.append(sc_x + sc_y)


def make_facet_plot(
    plot_df,
    plot_path,
    x_col,
    y_col=None,
    rename_x=None,
    x_order=None,
    x_palette=None,
    x_count=True,
    hue_col=None,
    rename_hue=None,
    hue_order=None,
    hue_palette=None,
    hue_count=True,
    row_col=None,
    rename_row=None,
    row_order=None,
    row_count=True,
    row_palette=None,
    col_col=None,
    rename_col=None,
    col_order=None,
    col_count=False,
    col_wrap=None,
    col_palette=None,
    plot_width=4,
    plot_height=None,
    font_size=7,
    marker_size=1,
    line_width=0.5,
    tick_len=2,
    show_legend=False,
    legend_pad=5,
    legend_cols=None,
    legend_marker_size=None,
    color_legend_text=False,
    highlight_palette=None,
    highlight_lst=None,
    darken_lst=None,
    darken_palette=None,
    x_lim=None,
    y_lim=None,
    x_ticks=None,
    y_ticks=None,
    x_round=0,
    y_round=0,
    x_rotation=0,
    y_rotation=0,
    x_ha="center",
    y_ha="right",
    x_str=None,
    y_str=None,
    x_pad=None,
    y_pad=None,
    h_lines=None,
    v_lines=None,
    h_color=None,
    v_color=None,
    count_chain=True,
    count_pdb=False,
    count_cf=False,
    plot_scatter=True,
    plot_line=False,
    plot_reg=False,
    plot_kde=False,
    plot_kind=None,
    log_reg=False,
    reg_estimator=None,
    reg_bins=None,
    trun_reg=False,
    kde_common_norm=True,
    kde_common_grid=False,
    kde_bw=1.0,
    kde_thresh=0.05,
    kde_levels=10,
    kde_cut=3.0,
    stat_pairs=None,
    stat_test="t-test_ind",
    stat_loc="inside",
    stat_format="star",
    tick_mult=0.75,
    all_ticks=False,
):
    df = plot_df.copy(deep=True)

    df_col_lst = list(df.columns)

    reformat_lst = list()
    if x_col in reformat_col_lst:
        if x_col not in df_col_lst:
            reformat_lst.append(x_col)
    if y_col in reformat_col_lst:
        if y_col is not None:
            if y_col not in df_col_lst:
                reformat_lst.append(y_col)

    if len(reformat_lst) > 0:
        df = reformat_val_table(df, reformat_lst)

    if plot_kind is None:
        df[x_col] = df[x_col].map(float)
    else:
        if type(x_palette) == str:
            x_palette = [x_palette] * len(lst_col(df, x_col))
        df, x_lst, x_palette = prep_plot_col(
            df,
            x_col,
            color_palette=x_palette,
            rename_vals=rename_x,
            order_lst=x_order,
            label_count=x_count,
            count_chain=count_chain,
            count_pdb=count_pdb,
            count_cf=count_cf,
        )

    if y_col is not None:
        df[y_col] = df[y_col].map(float)

    if hue_col is not None:
        df, hue_lst, hue_palette = prep_plot_col(
            df,
            hue_col,
            color_palette=hue_palette,
            rename_vals=rename_hue,
            order_lst=hue_order,
            label_count=hue_count,
            count_chain=count_chain,
            count_pdb=count_pdb,
            count_cf=count_cf,
        )
        if len(hue_lst) == 1:
            if type(show_legend) != dict:
                show_legend = False
    else:
        hue_lst = type_lst(0)
        if type(show_legend) != dict:
            show_legend = False

    if row_col is not None:
        df, row_lst, row_palette = prep_plot_col(
            df,
            row_col,
            color_palette=row_palette,
            rename_vals=rename_row,
            order_lst=row_order,
            label_count=row_count,
            count_chain=count_chain,
            count_pdb=count_pdb,
            count_cf=count_cf,
        )

        total_row = len(row_lst)

        if row_palette is not None:
            row_color_dict = get_lst_colors(
                row_lst, palette=row_palette, return_dict=True
            )

        if hue_col == row_col:
            if type(show_legend) != dict:
                show_legend = False
    else:
        row_lst = None
        total_row = 1

    title_col = True
    if col_col is None:
        col_col = resid_col
        if resid_col not in list(df.columns):
            df[resid_col] = "None"
            title_col = False

    df, col_lst, col_palette = prep_plot_col(
        df,
        col_col,
        color_palette=col_palette,
        rename_vals=rename_col,
        order_lst=col_order,
        label_count=col_count,
        count_chain=count_chain,
        count_pdb=count_pdb,
        count_cf=count_cf,
    )
    total_col = len(col_lst)

    if col_palette is not None:
        col_color_dict = get_lst_colors(col_lst, palette=col_palette, return_dict=True)

    df = mask_unequal(df, x_col, 999.00)
    if x_str is None:
        x_str = title_str(x_col)
    if x_col in bb_col_lst:
        x_lim = bb_lim
        if x_ticks is not False:
            x_ticks = bb_ticks
    elif x_col in sc_col_lst and x_lim is None:
        x_lim = sc_lim
        if x_ticks is not False:
            x_ticks = sc_ticks
    if y_col is not None:
        df = mask_unequal(df, y_col, 999.00)
        if y_str is None:
            y_str = title_str(y_col)
        if y_col in bb_col_lst:
            y_lim = bb_lim
            if y_ticks is not False:
                y_ticks = bb_ticks
        elif y_col in sc_col_lst and y_lim is None:
            y_lim = sc_lim
            if y_ticks is not False:
                y_ticks = sc_ticks
    else:
        if y_str is None:
            y_str = "Density"

    if x_lim is not None:
        if type(x_lim) != tuple:
            x_lim = tuple(x_lim)
    if y_lim is not None:
        if type(y_lim) != tuple:
            y_lim = tuple(y_lim)

    if plot_kind is None or hue_col is not None:
        hue_color_dict = get_lst_colors(hue_lst, palette=hue_palette, return_dict=True)
    else:
        hue_color_dict = get_lst_colors(x_lst, palette=x_palette, return_dict=True)
    if highlight_lst is not None:
        highlight_lst = format_nested_lst(highlight_lst, return_str=True)
        highlight_color_lst = get_lst_colors(highlight_lst, palette=highlight_palette)

    if darken_lst is not None:
        darken_lst = format_nested_lst(darken_lst, return_str=True)
        darken_color_lst = get_lst_colors(darken_lst, palette=darken_palette)

    sns.set_context("paper")
    sns.set_style("ticks")
    sns.set_palette(list(hue_color_dict.values()))

    bbox_extra_artists = tuple()

    if row_col is None:
        if col_wrap is None:
            col_wrap = total_col
        total_col = col_wrap
        total_row = total_col / col_wrap
        show_margins = False
    else:
        col_wrap = None
        show_margins = True

    if plot_kind is None:

        g = sns.FacetGrid(
            df,
            hue=hue_col,
            hue_order=hue_lst,
            row=row_col,
            row_order=row_lst,
            col=col_col,
            col_order=col_lst,
            col_wrap=col_wrap,
            margin_titles=show_margins,
        )

        if y_col is None:
            g.map(
                sns.distplot,
                x_col,
                kde=True,
                hist=False,
                rug=True,
                kde_kws={
                    "common_norm": kde_common_norm,
                    "common_grid": kde_common_grid,
                    "bw_adjust": kde_bw,
                    "thresh": kde_thresh,
                    "levels": kde_levels,
                    "cut": kde_cut,
                    "alpha": 0.1,
                    "fill": True,
                    "linewidth": line_width,
                },
                rug_kws={"lw": 0.05, "alpha": 0.5, "height": 0.05},
            )

        else:

            if plot_line:
                g.map(sns.lineplot, x_col, y_col)
            if plot_reg:
                g.map(
                    sns.regplot,
                    x_col,
                    y_col,
                    scatter_kws={
                        "s": marker_size,
                        "linewidth": 0,
                        "alpha": 0.75,
                    },
                    line_kws={"linewidth": line_width},
                    logx=log_reg,
                    x_estimator=reg_estimator,
                    x_bins=reg_bins,
                    truncate=trun_reg,
                )
            if plot_kde:
                g.map(
                    sns.kdeplot,
                    x_col,
                    y_col,
                    common_norm=kde_common_norm,
                    common_grid=kde_common_grid,
                    bw_adjust=kde_bw,
                    cut=kde_cut,
                    thresh=kde_thresh,
                    levels=kde_levels,
                    fill=True,
                    linewidth=0,
                    alpha=0.25,
                )

            if plot_scatter:
                g.map(
                    plt.scatter,
                    x_col,
                    y_col,
                    s=marker_size,
                    linewidth=0,
                    alpha=0.75,
                )
    else:
        if plot_kind == "strip" or plot_kind == "swarm" or plot_kind == "point":
            g = sns.catplot(
                data=df,
                x=x_col,
                y=y_col,
                hue=hue_col,
                row=row_col,
                col=col_col,
                col_wrap=col_wrap,
                order=x_lst,
                hue_order=hue_lst,
                row_order=row_lst,
                col_order=col_lst,
                kind=plot_kind,
                palette=hue_color_dict,
                legend=False,
                sharex=False,
                sharey=True,
                margin_titles=True,
                s=marker_size,
                linewidth=0,
                alpha=0.75,
                dodge=True,
            )
        else:
            if (
                row_col is not None
                or col_col is not resid_col
                or plot_kind == "count"
                or plot_kind == "bar"
            ):

                if plot_kind == "count":
                    cat_width = 0
                else:
                    cat_width = line_width

                g = sns.catplot(
                    data=df,
                    x=x_col,
                    y=y_col,
                    hue=hue_col,
                    row=row_col,
                    col=col_col,
                    col_wrap=col_wrap,
                    order=x_lst,
                    hue_order=hue_lst,
                    row_order=row_lst,
                    col_order=col_lst,
                    kind=plot_kind,
                    palette=hue_color_dict,
                    legend=False,
                    sharex=False,
                    sharey=True,
                    linewidth=cat_width,
                    margin_titles=True,
                )
            else:
                if plot_kind == "box" or plot_kind == "boxen" or plot_kind == "violin":

                    g = sns.catplot(
                        data=df,
                        x=x_col,
                        y=y_col,
                        hue=hue_col,
                        order=x_lst,
                        hue_order=hue_lst,
                        kind=plot_kind,
                        color="white",
                        legend=False,
                        linewidth=line_width,
                        showfliers=False,
                    )

                    ax = sns.stripplot(
                        data=df,
                        x=x_col,
                        y=y_col,
                        hue=hue_col,
                        order=x_lst,
                        hue_order=hue_lst,
                        palette=hue_color_dict,
                        s=marker_size,
                        alpha=0.75,
                        split=True,
                    )
                    if ax.get_legend() is not None:
                        ax.get_legend().remove()

                    if stat_pairs is not None:
                        x_dict = dict()
                        for val in x_lst:
                            x_dict[val.split(" (N=")[0]] = val

                        box_pairs = list()
                        for i, stat in enumerate(stat_pairs):
                            box_pairs.append((x_dict[stat[0]], x_dict[stat[1]]))

                        add_stat_annotation(
                            ax,
                            data=df,
                            x=x_col,
                            y=y_col,
                            order=x_lst,
                            box_pairs=box_pairs,
                            test=stat_test,
                            text_format=stat_format,
                            loc=stat_loc,
                            verbose=2,
                            fontsize=font_size,
                            linewidth=line_width,
                        )

    if plot_height is None:
        plot_height = plot_width * (total_row / total_col)

    g.fig.set_figheight(plot_height)
    g.fig.set_figwidth(plot_width)

    if plot_kind is None:
        if (x_ticks is None or x_ticks is False) and (
            y_ticks is None or y_ticks is False
        ):
            g.set(
                xlim=x_lim,
                ylim=y_lim,
            )
        elif x_ticks is None or x_ticks is False:
            g.set(
                xlim=x_lim,
                ylim=y_lim,
                yticks=y_ticks,
            )
        elif y_ticks is None or y_ticks is False:
            g.set(
                xlim=x_lim,
                ylim=y_lim,
                xticks=x_ticks,
            )
        else:
            g.set(
                xlim=x_lim,
                ylim=y_lim,
                xticks=x_ticks,
                yticks=y_ticks,
            )
    else:
        if y_ticks is not None and y_lim is not None:
            g.set(
                ylim=y_lim,
                yticks=y_ticks,
            )
        elif y_ticks is not None:
            g.set(
                yticks=y_ticks,
            )
        elif y_lim is not None:
            g.set(
                ylim=y_lim,
            )

    g.fig.subplots_adjust(wspace=0, hspace=0)

    g.fig.add_subplot(111, frameon=False)
    plt.tick_params(labelcolor="none", top=False, bottom=False, left=False, right=False)

    x_label = plt.xlabel(x_str, fontsize=font_size, labelpad=x_pad)
    y_label = plt.ylabel(y_str, fontsize=font_size, labelpad=y_pad)

    bbox_extra_artists += (x_label, y_label)

    if show_legend or type(show_legend) == dict:

        if legend_cols is None:
            legend_cols = get_ncols(hue_lst)

        if legend_marker_size is None:
            legend_marker_size = marker_size * 3

        if type(show_legend) == dict:
            handles = [
                plt.plot(
                    [],
                    [],
                    marker="o",
                    ls="",
                    markersize=legend_marker_size,
                    markeredgewidth=0,
                    markerfacecolor=show_legend[hue],
                    label=hue,
                    color=show_legend[hue],
                )[0]
                for hue in list(show_legend.keys())
            ]
        else:
            handles = [
                plt.plot(
                    [],
                    [],
                    marker="o",
                    ls="",
                    markersize=legend_marker_size,
                    markeredgewidth=0,
                    markerfacecolor=hue_color_dict[hue],
                    label=hue,
                    color=hue_color_dict[hue],
                )[0]
                for hue in hue_lst
            ]

        legend_label_color = None
        if color_legend_text:
            legend_label_color = "linecolor"

        legend = g.fig.legend(
            handles=handles,
            fontsize=font_size,
            ncol=legend_cols,
            loc="upper center",
            frameon=False,
            bbox_transform=g.fig.transFigure,
            bbox_to_anchor=(0.5, 0),
            borderaxespad=legend_pad,
            labelcolor=legend_label_color,
        )

        bbox_extra_artists += (legend,)

    g.set_axis_labels("", "")

    tick_index = 0

    total_axes = len(col_lst)
    if row_col is not None:
        total_axes *= len(row_lst)

    ax_lst = col_lst
    if row_col is not None and col_col is None:
        ax_lst = row_lst
    elif row_col is not None and col_col is not None:
        ax_lst = list()
        for row in row_lst:
            for col in col_lst:
                ax_lst.append((row, col))

    if h_lines is not None:
        h_line_lst = type_lst(h_lines)
        if h_color is None:
            h_color = "black"

    if v_lines is not None:
        v_line_lst = type_lst(v_lines)
        if v_color is None:
            v_color = "black"

    for i, ax in enumerate(g.axes.flat):

        ax_name = ax_lst[i]

        if h_lines is not None:
            for h in h_line_lst:
                ax.axhline(y=h, linewidth=line_width, color=h_color)

        if v_lines is not None:
            for v in v_line_lst:
                ax.axvline(x=v, linewidth=line_width, color=v_color)

        clean_name = ax_name
        if type(clean_name) == str:
            if " (N=" in clean_name:
                clean_name = clean_name.split(" (N=")[0]
        elif type(clean_name) == tuple:
            clean_name = list(clean_name)
            for n, name in enumerate(clean_name):
                clean_name[n] = name.split(" (N=")[0]
            clean_name = tuple(clean_name)

        ax_grid_hex = grid_hex
        if darken_lst is not None:
            for d, darken in enumerate(darken_lst):
                if clean_name in darken:
                    ax.set_facecolor(darken_color_lst[d])
                    ax_grid_hex = "white"

        if x_col == phi_col and y_col == psi_col:
            for bb_grid in bb_grid_lst:
                ax.fill_between(
                    bb_grid[0],
                    bb_grid[1],
                    bb_grid[2],
                    facecolor="none",
                    edgecolor=ax_grid_hex,
                    linewidth=line_width,
                    zorder=0,
                )
        elif (x_col in sc_col_lst and y_col in sc_col_lst) and (
            x_ticks is None and y_ticks is None
        ):
            for sc_grid in sc_grid_lst:
                ax.fill_between(
                    sc_grid[0],
                    sc_grid[1],
                    sc_grid[2],
                    facecolor="none",
                    edgecolor=ax_grid_hex,
                    linewidth=line_width,
                    zorder=0,
                )
        else:
            if plot_kind is None:
                ax.xaxis.grid(color=ax_grid_hex, linewidth=line_width)
            ax.yaxis.grid(color=ax_grid_hex, linewidth=line_width)

        ax.set_axisbelow(True)

        if row_col is None:
            if ax.get_title():
                ax.set_title("")

            if len(col_lst) > 1:

                highlight_color = gray_hex
                if highlight_lst is not None:
                    for h, highlight in enumerate(highlight_lst):
                        if clean_name in highlight:
                            highlight_color = highlight_color_lst[h]

                divider = make_axes_locatable(ax)
                cax = divider.append_axes("top", size="20%", pad=0)
                cax.get_xaxis().set_visible(False)
                cax.get_yaxis().set_visible(False)
                cax.set_facecolor(highlight_color)
                for _, spine in cax.spines.items():
                    spine.set_visible(True)
                    spine.set_color("black")
                    spine.set_linewidth(line_width)
                cax.text(
                    0.5,
                    0.4,
                    str(ax_name),
                    color="white",
                    fontsize=font_size * 0.75,
                    verticalalignment="center",
                    horizontalalignment="center",
                    transform=cax.transAxes,
                )
        else:
            if ax.get_title():

                if title_col:
                    col = ax.get_title().split("=", 1)[1]
                else:
                    col = ""

                col_color = "black"
                if col_palette is not None:
                    col_color = col_color_dict[col]

                col_txt = ax.set_title(
                    col,
                    fontsize=font_size,
                    color=col_color,
                )

                bbox_extra_artists += (col_txt,)

            if ax.texts:
                row = ax.texts[0].get_text().split("=", 1)[1]

                row_color = "black"
                if row_palette is not None:
                    row_color = row_color_dict[row[1:]]

                row_txt = ax.text(
                    ax.texts[0].get_unitless_position()[0],
                    ax.texts[0].get_unitless_position()[1],
                    row,
                    transform=ax.transAxes,
                    va="center",
                    fontsize=font_size,
                    color=row_color,
                )
                ax.texts[0].remove()

                bbox_extra_artists += (row_txt,)

        add_ticks = False
        bottom_label = True
        left_label = True
        bottom = True
        left = True

        if total_col == 1:
            add_ticks = True
            if i != total_axes - 1:
                bottom_label = False
                bottom = False
        elif total_axes == 1 and row_col is None:
            add_ticks = True
        elif total_axes == col_wrap and row_col is None:
            add_ticks = True
            if i != 0:
                left_label = False
                left = False
        elif i % total_col == 0:
            if tick_index % 2 != 0 or all_ticks:
                add_ticks = True
            tick_index += 1
            if not all_ticks:
                bottom_label = False
                bottom = False
        elif (total_axes - i) < total_col + 1:
            if i % 2 != 0 or all_ticks:
                add_ticks = True
            left_label = False
            left = False
        else:
            bottom_label = False
            left_label = False
            bottom = False
            left = False

        if add_ticks:

            tick_format = "{:,."

            if plot_kind is None:
                x_tick_format = tick_format + str(x_round)

                x_tick_format += "f}"

                x_tick_lst = ax.get_xticks()
                ax.xaxis.set_major_locator(mticker.FixedLocator(x_tick_lst))
                ax.set_xticklabels(
                    [x_tick_format.format(x) for x in x_tick_lst],
                    fontsize=font_size * tick_mult,
                    rotation=x_rotation,
                    ha=x_ha,
                )
            else:
                ax.set_xticklabels(
                    ax.get_xticklabels(),
                    fontsize=font_size * tick_mult,
                    rotation=45,
                    ha="right",
                )

            y_tick_format = tick_format + str(y_round)
            y_tick_format += "f}"

            y_tick_lst = ax.get_yticks()
            ax.yaxis.set_major_locator(mticker.FixedLocator(y_tick_lst))
            ax.set_yticklabels(
                [y_tick_format.format(y) for y in y_tick_lst],
                fontsize=font_size * tick_mult,
                rotation=y_rotation,
                ha=y_ha,
            )

        elif not add_ticks:
            bottom_label = False
            left_label = False

        if x_ticks is False:
            bottom_label = False
            bottom = False
        if y_ticks is False:
            left = False
            left_label = False

        ax.tick_params(
            direction="out",
            labelbottom=bottom_label,
            labelleft=left_label,
            bottom=bottom,
            left=left,
            length=tick_len,
            width=line_width,
            colors="black",
        )

        for _, spine in ax.spines.items():
            spine.set_visible(True)
            spine.set_color("black")
            spine.set_linewidth(line_width)

    append_file_path(plot_path)

    plt.savefig(
        plot_path,
        format="pdf",
        bbox_extra_artists=bbox_extra_artists,
        bbox_inches="tight",
        pad_inches=0.0,
        dpi=600,
    )
    plt.close()

    print("Made facet plot!")



# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import streamlit as st

from ..functions import *
from ..tools import *


def app():

    st.markdown(
        """
        # Summary
        *rascore* is a tool for analyzing RAS structures 
        (KRAS, NRAS, and HRAS) by the conformations of their 
        catalytic switch 1 (SW1) and switch 2 (SW2) loops. 
        In addition, *rascore* can be used to build and query 
        an updatable database of all available RAS structures 
        from the Protein Data Bank with their SW1 and SW2 loops 
        conformationally classified and their molecular contents 
        annotated (e.g., mutation status, nucleotide state, bound 
        proteins, small molecule inhibitors, etc.). 

        Details of our RAS conformational classification are 
        provided on *BioArxiv* in the manuscript: **An expanded 
        classification of active, inactive and druggable RAS 
        conformations.** We hope that researchers will use 
        *rascore* to gain novel insights into RAS biology and 
        drug discovery. 

        # Usage

        To the left, there is a sidebar with a dropdown menu
        for navigating to each page in the *rascore* GUI application:

        - Home
        - Search for PDB Entry
        - Query Database
        - Classify RAS Structure(s)

        # Authors
        Please feel free to contact us with any issues, comments, or questions.

        ### Mitchell Parker

        - Email: <mitch.isaac.parker@gmail.com>
        - GitHub: https://github.com/mitch-parker

        ### Roland Dunbrack

        - Email: <roland.dunbrack@gmail.com>
        - GitHub: https://github.com/DunbrackLab

        # Funding

        - NIH F30 GM142263 (to M.P.)
        - NIH R35 GM122517 (to R.D.)

        # License
        MIT License

        Copyright (c) 2022 Mitchell Isaac Parker
        """
    )



# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import pandas as pd
from tqdm import tqdm

from ..functions import *


def search_pdbaa(
    pdbaa_fasta_path,
    search_lst,
    entry_table_path=None,
    min_length=None,
    min_resolution=None,
):

    pdbaa_dict = load_record_dict(pdbaa_fasta_path)

    for search in search_lst:
        if "_" in search:
            search_col = swiss_id_col
            break
        else:
            search_col = pdb_id_col
            break

    col_lst = [
        pdb_id_col,
        pdb_code_col,
        chainid_col,
        swiss_id_col,
        prot_col,
        method_col,
        resolution_col,
        r_factor_col,
        seq_col,
        len_col,
    ]

    record_dict = {}

    for col in col_lst:
        record_dict[col] = list()

    for record in tqdm(
        list(pdbaa_dict.keys()), desc="Searching pdbaa", position=0, leave=True
    ):

        record = pdbaa_dict[record]

        desc = get_record_desc(record)

        swiss_id = get_pdbaa_swiss_id(desc)

        pdb_code = get_pdbaa_pdb_code(desc)
        chainid = get_pdbaa_chainid(desc)
        pdb_id = get_pdb_id(pdb_code, chainid)

        get_seq = False

        if search_col == swiss_id_col:
            if swiss_id in search_lst:
                get_seq = True

        elif search_col == pdb_id_col:
            if pdb_id in search_lst:
                get_seq = True

        if get_seq:

            seq = get_record_seq(record)
            length = len(seq)

            get_lst = True
            if min_length is not None:
                if length <= min_length:
                    get_lst = False

            if get_lst:

                resolution = get_pdbaa_resolution(desc)

                add_dict = True
                if min_resolution is not None:
                    if resolution <= min_resolution:
                        add_dict = False

                if add_dict:

                    record_dict[pdb_id_col].append(pdb_id)
                    record_dict[pdb_code_col].append(pdb_code)
                    record_dict[chainid_col].append(chainid)
                    record_dict[method_col].append(get_pdbaa_method(desc))
                    record_dict[resolution_col].append(resolution)
                    record_dict[r_factor_col].append(get_pdbaa_r_factor(desc))
                    record_dict[swiss_id_col].append(swiss_id)
                    record_dict[prot_col].append(get_pdbaa_prot(desc))
                    record_dict[seq_col].append(seq)
                    record_dict[len_col].append(length)

    df = pd.DataFrame(record_dict)

    total_entry = len(lst_col(df, pdb_code_col, unique=True))
    total_chains = len(lst_col(df, pdb_id_col, unique=True))

    print(
        f"Your search identified {total_chains} chains from {total_entry} PDB entries!"
    )

    if entry_table_path is not None:
        save_table(entry_table_path, df)
    else:
        return df



# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import pandas as pd
from tqdm import tqdm
import concurrent.futures

from ..functions import *


def build_lig_df(
    df,
    index,
    lig_dir=None,
    site_dict=None,
    match_dict=None,
    other_site="Other",
    other_match="Unclassified",
    none_site="None",
    none_match="None",
    max_site_dist=4,
    coord_path_col=None,
):

    if coord_path_col is None:
        coord_path_col = core_path_col

    df = get_df_at_index(df, index)

    coord_path = df.at[index, coord_path_col]
    modelid = df.at[index, modelid_col]
    chainid = df.at[index, chainid_col]

    lig_lst_dict = dict()

    for lig_col in lig_col_lst:

        lig_lst_dict[lig_col] = list()

    structure = load_coord(coord_path)
    neighbors = get_neighbors(structure[fix_val(modelid, return_int=True)][chainid])

    bio_cont_lst = list()
    pharm_cont_lst = list()
    site_lst = list()
    match_lst = list()

    for residue in get_residues(structure):
        if is_het(residue):
            cont_lst = lst_unique(
                [
                    get_resnum(x)
                    for x in get_residue_cont(
                        neighbors,
                        residue,
                        max_dist=max_site_dist,
                        level="R",
                    )
                    if (get_resnum(x) < 50000)
                    and (get_reschainid(x) == chainid)
                    and (is_aa(x))
                ]
            )
            if len(cont_lst) > 0:
                resname = get_resname(residue)
                lig_class = False
                is_bio = False
                is_pharm = False
                for lig_col in lig_col_lst:
                    if resname in lig_class_dict[lig_col]:
                        lig_class = True
                        if resname not in lig_lst_dict[lig_col]:
                            lig_lst_dict[lig_col].append(resname)
                            if lig_col == bio_lig_col:
                                is_bio = True
                            if lig_col == pharm_lig_col:
                                is_pharm = True

                if not lig_class:
                    if resname not in lig_lst_dict[pharm_lig_col]:
                        lig_lst_dict[pharm_lig_col].append(resname)
                        is_pharm = True

                if is_bio:
                    bio_cont_lst += cont_lst

                if is_pharm:
                    pharm_cont_lst += cont_lst

                    if site_dict is not None:

                        max_cont = 0
                        site_status = other_site
                        for site_name, site_cont_lst in site_dict.items():
                            site_cont = len(
                                [x for x in site_cont_lst if x in pharm_cont_lst]
                            )
                            if site_cont > max_cont:
                                site_status = site_name
                                max_cont = site_cont
                        site_lst.append(site_status)

                    if match_dict is not None:
                        if site_status in list(match_dict.keys()):
                            match_status = other_match
                            match_name_lst = list()
                            match_simi_lst = list()
                            for match_name, query_lst in match_dict[
                                site_status
                            ].items():
                                is_match = is_lig_match(
                                    resname,
                                    query_lst,
                                )
                                if is_match:
                                    for query in query_lst:
                                        match_name_lst.append(match_name)
                                        match_simi_lst.append(
                                            get_lig_simi(
                                                resname, query, lig_dir=lig_dir
                                            )
                                        )
                            if len(match_name_lst) > 0:
                                match_status = match_name_lst[
                                    match_simi_lst.index(max(match_simi_lst))
                                ]
                            match_lst.append(match_status)

    for lig_col in lig_col_lst:
        df.at[index, lig_col] = lst_to_str(lig_lst_dict[lig_col])

    df.at[index, pharm_lig_site_col] = lst_to_str(lst_unique(site_lst), empty=none_site)
    df.at[index, pharm_lig_match_col] = lst_to_str(
        lst_unique(match_lst), empty=none_match
    )
    df.at[index, bio_lig_cont_col] = lst_to_str(lst_unique(bio_cont_lst))

    df.at[index, pharm_lig_cont_col] = lst_to_str(
        lst_unique(pharm_cont_lst), empty=none_match
    )

    return df


def annot_lig(
    df,
    lig_table_path=None,
    lig_dir=None,
    site_dict=None,
    match_dict=None,
    other_site="Other",
    other_match="Unclassified",
    none_site="None",
    none_match="None",
    max_site_dist=4,
    coord_path_col=None,
    num_cpu=1,
):

    if site_dict is not None:
        for site_name, site_cont in site_dict.items():
            site_dict[site_name] = res_to_lst(site_cont)

    if match_dict is not None:
        for site_name in list(match_dict.keys()):
            for match_name, query_lst in match_dict[site_name].items():
                match_dict[site_name][match_name] = type_lst(query_lst)

    lig_df = pd.DataFrame()

    if num_cpu == 1:
        for index in tqdm(
            list(df.index.values), desc="Annotating ligands", position=0, leave=True
        ):

            lig_df = pd.concat(
                [
                    lig_df,
                    build_lig_df(
                        df,
                        index,
                        lig_dir=lig_dir,
                        site_dict=site_dict,
                        match_dict=match_dict,
                        other_site=other_site,
                        other_match=other_match,
                        none_site=none_site,
                        none_match=none_match,
                        max_site_dist=max_site_dist,
                        coord_path_col=coord_path_col,
                    ),
                ],
                sort=False,
            )
    else:
        with concurrent.futures.ProcessPoolExecutor(max_workers=num_cpu) as executor:
            job_lst = [
                executor.submit(
                    build_lig_df,
                    df,
                    index,
                    site_dict=site_dict,
                    match_dict=match_dict,
                    other_site=other_site,
                    other_match=other_match,
                    none_site=none_site,
                    none_match=none_match,
                    max_site_dist=max_site_dist,
                    coord_path_col=coord_path_col,
                )
                for index in list(df.index.values)
            ]

            for job in tqdm(
                concurrent.futures.as_completed(job_lst),
                desc="Annotating ligands",
                total=len(job_lst),
                miniters=1,
                position=0,
                leave=True,
            ):

                lig_df = pd.concat([lig_df, job.result()], sort=False)

    lig_df = lig_df.reset_index(drop=True)

    print("Annotated ligands!")

    if lig_table_path is not None:
        save_table(lig_table_path, lig_df)
    else:
        return lig_df



# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import pandas as pd
from tqdm import tqdm

from ..functions import *


def calc_interf_cont_dist(
    coord_path,
    chainid,
    interf,
    interf_dict,
    cont_dist=5,
    modelid=None,
    interf_resid_lst=None,
):

    if modelid is None:
        modelid = 0

    interf_path = interf_dict[coord_path][chainid][interf][interf_path_col]

    structure = load_coord(interf_path)

    neighbors = get_neighbors(structure)

    resid_lst = str_to_lst(interf_dict[coord_path][chainid][interf][atomid_cont_col])

    interf_cont_lst = list()
    cb_dist_lst = list()

    add_interf = True
    if interf_resid_lst is not None:
        add_interf = False

    for resid in resid_lst:

        if interf_resid_lst is not None:
            if extract_int(resid) in interf_resid_lst:
                add_interf = True

        if resid is not None and add_interf:

            if has_resid(structure, chainid, resid_to_tuple(resid), modelid=modelid):
                residue = structure[fix_val(modelid, return_int=True)][chainid][
                    resid_to_tuple(resid)
                ]

                resname = get_resname(residue)

                if resname == "GLY":
                    atomid = "CA"
                else:
                    atomid = "CB"

                for cont_residue in get_residue_cont(
                    neighbors, residue, max_dist=cont_dist, level="R"
                ):
                    cont_chainid = get_reschainid(cont_residue)
                    cont_resid = get_resnum(cont_residue)
                    cont_resname = get_resname(cont_residue)

                    if is_aa(cont_residue) and cont_chainid != chainid:

                        if cont_resname == "GLY":
                            cont_atomid = "CA"
                        else:
                            cont_atomid = "CB"

                        if cont_resid is not None:

                            interf_cont = lst_to_str(
                                sort_lst([resid, cont_resid]),
                                join_txt=":",
                            )

                            if interf_cont not in interf_cont_lst:

                                cb_dist = calc_atom_dist(
                                    structure,
                                    modelid_1=modelid,
                                    chainid_1=chainid,
                                    resid_1=resid,
                                    atomid_1=atomid,
                                    modelid_2=modelid,
                                    chainid_2=cont_chainid,
                                    resid_2=cont_resid,
                                    atomid_2=cont_atomid,
                                )

                                interf_cont_lst.append(interf_cont)
                                cb_dist_lst.append(cb_dist)

    return interf_cont_lst, cb_dist_lst


def get_index_interf(
    df,
    index,
    interf_dict,
    interf_resid_lst=None,
    min_area=200,
    cont_dist=5,
    iso_interf=False,
    het_interf=False,
    search_interf_cont_lst=None,
    search_cb_dist_lst=None,
    search_max_dist=0.7,
    coord_path_col=None,
):

    if coord_path_col is None:
        coord_path_col = renum_path_col

    index_df = get_df_at_index(df, index)

    coord_path = index_df.at[index, coord_path_col]
    modelid = index_df.at[index, modelid_col]
    chainid = index_df.at[index, chainid_col]

    interf_df = pd.DataFrame()

    for interf in list(interf_dict[coord_path][chainid].keys()):

        add_interf = True

        if float(interf_dict[coord_path][chainid][interf][interf_area_col]) < min_area:
            add_interf = False

        interf_iso_status = interf_dict[coord_path][chainid][interf][iso_col]
        if iso_interf:
            if not interf_iso_status:
                add_interf = False
        if het_interf:
            if interf_iso_status:
                add_interf = False

        if add_interf:

            interf_cont_lst, cb_dist_lst = calc_interf_cont_dist(
                coord_path,
                chainid,
                interf,
                interf_dict,
                cont_dist=cont_dist,
                modelid=modelid,
                interf_resid_lst=interf_resid_lst,
            )

            if search_interf_cont_lst is not None and search_cb_dist_lst is not None:
                if (
                    calc_q_score(
                        i_cont_lst=search_interf_cont_lst,
                        j_cont_lst=interf_cont_lst,
                        i_dist_lst=search_cb_dist_lst,
                        j_dist_lst=cb_dist_lst,
                    )
                    > search_max_dist
                ):
                    add_interf = False

        if add_interf:
            temp_df = index_df.copy(deep=True)

            for col in list(interf_dict[coord_path][chainid][interf].keys()):
                temp_df.at[index, col] = interf_dict[coord_path][chainid][interf][col]

            temp_df.at[index, interf_col] = interf
            temp_df.at[index, interf_cont_col] = lst_to_str(interf_cont_lst)
            temp_df.at[index, cb_dist_col] = lst_to_str(cb_dist_lst)

            interf_df = pd.concat([interf_df, temp_df], sort=False)

    return interf_df


def build_interf_table(
    df,
    interf_dict,
    interf_table_path=None,
    interf_resids=None,
    min_area=200,
    cont_dist=5,
    iso_interf=False,
    het_interf=False,
    search_coord_path=None,
    search_modelid=None,
    search_chainid=None,
    search_interf=None,
    search_max_dist=0.7,
    coord_path_col=None,
):

    if coord_path_col is None:
        coord_path_col = renum_path_col

    interf_df = pd.DataFrame()

    interf_resid_lst = res_to_lst(interf_resids)

    search_interf_cont_lst = None
    search_cb_dist_lst = None

    if (
        search_coord_path is not None
        and search_chainid is not None
        and search_interf is not None
    ):

        try:
            search_interf_cont_lst, search_cb_dist_lst = calc_interf_cont_dist(
                search_coord_path,
                search_chainid,
                str(search_interf),
                interf_dict,
                cont_dist=cont_dist,
                modelid=search_modelid,
                interf_resid_lst=interf_resid_lst,
            )
        except:
            search_interf_cont_lst, search_cb_dist_lst = calc_interf_cont_dist(
                search_coord_path,
                search_chainid,
                int(search_interf),
                interf_dict,
                cont_dist=cont_dist,
                modelid=search_modelid,
                interf_resid_lst=interf_resid_lst,
            )

    for index in tqdm(
        list(df.index.values), desc="Building interface table", position=0, leave=True
    ):

        interf_df = pd.concat(
            [
                interf_df,
                get_index_interf(
                    df,
                    index,
                    interf_dict,
                    min_area=min_area,
                    cont_dist=cont_dist,
                    interf_resid_lst=interf_resid_lst,
                    iso_interf=iso_interf,
                    het_interf=het_interf,
                    search_interf_cont_lst=search_interf_cont_lst,
                    search_cb_dist_lst=search_cb_dist_lst,
                    search_max_dist=search_max_dist,
                    coord_path_col=coord_path_col,
                ),
            ],
            sort=False,
        )

    interf_df = interf_df.reset_index(drop=True)

    interf_df[interf_col] = interf_df[interf_col].map(str)
    interf_df = fix_col(interf_df, interf_col)

    interf_df[interf_id_col] = interf_df[pdb_id_col] + interf_df[interf_col]

    if interf_table_path is not None:
        save_table(interf_table_path, interf_df)
    else:
        return interf_df

    print("Built interface table!")



# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import pandas as pd
from tqdm import tqdm

from ..functions import *


def classify_matrix(
    cluster_df,
    pred_df,
    fit_matrix,
    pred_matrix,
    result_table_path,
    sum_table_path=None,
    report_table_path=None,
    max_nn_dist=None,
    fit_constr_matrix=None,
    pred_constr_matrix=None,
    max_constr_dist=None,
    constr_method="mean",
    only_save_pred=False,
    reorder_class=True,
    unclass_name="Noise",
):

    if max_nn_dist is None:
        nn_cutoff_dict = build_cutoff_dict(cluster_df, fit_matrix, method="min")
    else:
        nn_cutoff_dict = dict()

    if fit_constr_matrix is not None and pred_constr_matrix is not None:
        if max_constr_dist is None:
            constr_cutoff_dict = build_cutoff_dict(
                cluster_df, fit_constr_matrix, method=constr_method
            )
        else:
            constr_cutoff_dict = dict()

    cluster_lst = lst_col(cluster_df, cluster_col, unique=True, return_str=True)

    report_dict = dict()
    for cluster in cluster_lst:
        report_dict[cluster] = {
            total_classified_nn_col: 0,
            total_classified_constr_col: 0,
        }

    for index in tqdm(
        list(pred_df.index.values),
        desc="Classifying matrix",
        position=0,
        leave=True,
    ):

        pred_lst = list()

        nn_dict = dict()
        constr_dict = dict()

        classified_dict = dict()

        for cluster in cluster_lst:

            add_cluster = True

            index_lst = get_val_index_lst(cluster_df, cluster_col, cluster)

            if len(index_lst) == 1:
                nn_dist = 0
            else:
                if len(pred_df) < 2:
                    dist_lst = pred_matrix[index_lst]
                else:
                    dist_lst = pred_matrix[index, index_lst]
                nn_dist = calc_dist_stat(
                    dist_lst,
                    method="min",
                )

            nn_dict[cluster] = nn_dist

            nn_cutoff = max_nn_dist
            if max_nn_dist is None:
                nn_cutoff = nn_cutoff_dict[cluster]
            else:
                nn_cutoff_dict[cluster] = nn_cutoff

            if nn_dist > nn_cutoff:
                add_cluster = False
            else:
                classified_dict[cluster] = total_classified_nn_col

            if (
                fit_constr_matrix is not None and pred_constr_matrix is not None
            ) and add_cluster is False:
                if len(index_lst) == 1:
                    constr_dist = 0
                else:
                    constr_dist = calc_dist_stat(
                        pred_constr_matrix[index, index_lst],
                        method=constr_method,
                    )
                constr_dict[cluster] = constr_dist

                constr_cutoff = max_constr_dist
                if max_constr_dist is None:
                    constr_cutoff = constr_cutoff_dict[cluster]
                else:
                    constr_cutoff_dict[cluster] = constr_cutoff

                if constr_dist > constr_cutoff:
                    add_cluster = False
                else:
                    classified_dict[cluster] = total_classified_constr_col

            if add_cluster and cluster != unclass_name:
                pred_lst.append(cluster)

        pred_cluster = unclass_name

        if len(pred_lst) == 1:
            pred_cluster = pred_lst[0]
            report_dict[pred_cluster][classified_dict[pred_cluster]] += 1

        pred_df.at[index, cluster_col] = pred_cluster

        if pred_cluster in list(nn_dict.keys()):
            nn_dist = nn_dict[pred_cluster]
        else:
            nn_dist = 0

        pred_df.at[index, nn_dist_col] = nn_dist

        if fit_constr_matrix is not None and pred_constr_matrix is not None:
            if pred_cluster in list(constr_dict.keys()):
                constr_dist = constr_dict[pred_cluster]
            else:
                constr_dist = 0
            pred_df.at[index, constr_dist_col] = constr_dist

    if not only_save_pred:
        pred_df = pd.concat([cluster_df, pred_df], sort=False)
        pred_df = pred_df.reset_index(drop=True)
        if reorder_class:
            pred_df = order_clusters(pred_df)

    save_table(result_table_path, pred_df)

    if sum_table_path is not None:
        sum_df = build_sum_table(pred_df)
        save_table(sum_table_path, sum_df)

    if report_table_path is not None:
        cluster_count_dict = build_col_count_dict(pred_df, cluster_col)
        report_df = pd.DataFrame()
        for i, cluster in enumerate(
            [x for x in cluster_lst if x in list(cluster_count_dict.keys())]
        ):
            report_df.at[i, cluster_col] = cluster
            report_df.at[i, total_col] = cluster_count_dict[cluster]
            report_df.at[i, nn_cutoff_col] = nn_cutoff_dict[cluster]
            report_df.at[i, total_classified_nn_col] = report_dict[cluster][
                total_classified_nn_col
            ]
            if fit_constr_matrix is not None and pred_constr_matrix is not None:
                report_df.at[i, constr_cutoff_col] = constr_cutoff_dict[cluster]
                report_df.at[i, total_classified_constr_col] = report_dict[cluster][
                    total_classified_constr_col
                ]
        save_table(report_table_path, report_df)

    print("Classified matrix!")



# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from .tools import *
from .functions import *
from .constants import *


def classify_rascore(coord_paths, out_path=None, dih_dict=None, num_cpu=1):

    if out_path is None:
        out_path = f"{os.getcwd()}/{rascore_str}_{classify_str}"

    cluster_path = f"{get_dir_path(dir_str=data_str, dir_path=get_dir_name(__file__))}/{rascore_str}_{cluster_str}"

    coord_path_lst = type_lst(coord_paths)

    if type(coord_path_lst[0]) == DataFrame:
        df = coord_path_lst[0]
    else:
        df = pd.DataFrame()
        if ".txt" in coord_path_lst[0]:
            df = load_table(coord_path_lst[0])
            df_col_lst = list(df.columns)
            if core_path_col in df_col_lst and chainid_col in df_col_lst:
                if modelid_col not in df_col_lst:
                    df[modelid_col] = 0
            else:
                if core_path_col in df_col_lst and chainid_col not in df_col_lst:
                    coord_path_lst = lst_col(df, core_path_col)
                else:
                    coord_path_lst = load_lst(coord_path_lst[0])
                df = pd.DataFrame()

        if len(df) == 0:

            i = 0
            for coord_path in tqdm(
                coord_path_lst,
                desc="Loading coordinates",
                position=0,
                leave=True,
            ):
                structure = load_coord(coord_path)
                for model in structure:
                    modelid = get_modelid(model)
                    for chain in model:
                        chainid = get_chainid(chain)
                        df.at[i, core_path_col] = coord_path
                        df.at[i, modelid_col] = modelid
                        df.at[i, chainid_col] = chainid
                        i += 1

    if len(df) > 0:
        df_col_lst = list(df.columns)

        if id_col not in df_col_lst and pdb_id_col not in df_col_lst:
            for index in list(df.index.values):
                df.at[index, id_col] = get_file_name(df.at[index, core_path_col])

        missing_col_lst = [
            x for x in [core_path_col, modelid_col, chainid_col] if x not in df_col_lst
        ]
        if len(missing_col_lst) > 0:
            print(f"Input table missing columns: {lst_to_str(missing_col_lst)}")

        else:
            if len([x for x in lig_col_lst if x not in df_col_lst]) > 0:
                df = annot_lig(
                    df=df,
                    site_dict=pharm_site_dict,
                    match_dict=pharm_match_dict,
                    num_cpu=num_cpu,
                )

            if pharm_class_col not in df_col_lst:
                for index in list(df.index.values):
                    pharm_class = df.at[index, pharm_lig_site_col]
                    if pharm_class not in [sp2_name, sp12_name, none_pharm_name]:
                        pharm_class = other_pharm_name
                    df.at[index, pharm_class_col] = pharm_class

            if nuc_class_col not in df_col_lst:
                df[nuc_class_col] = df[bio_lig_col].map(nuc_class_dict).fillna(gtp_name)

            if dih_dict is None:
                dih_dict = prep_dih(coord_paths, num_cpu=num_cpu)

            for loop_name, loop_resids in loop_resid_dict.items():

                cluster_loop_path = f"{cluster_path}/{loop_name}"
                classify_loop_path = f"{out_path}/{loop_name}"

                loop_result_df = pd.DataFrame()
                loop_sum_df = pd.DataFrame()
                loop_classify_report_df = pd.DataFrame()

                for nuc_class in nuc_class_lst:

                    print(
                        f"Classifying {loop_name} conformations in {nuc_class} structures."
                    )

                    loop_nuc_name = f"{loop_name}_{nuc_class}"

                    cluster_table_path = get_file_path(
                        cluster_table_file,
                        dir_str=loop_nuc_name,
                        dir_path=cluster_loop_path,
                    )

                    fit_matrix_path = get_file_path(
                        dih_fit_matrix_file,
                        dir_str=loop_nuc_name,
                        dir_path=cluster_loop_path,
                    )

                    result_table_path = get_file_path(
                        result_table_file,
                        dir_str=loop_nuc_name,
                        dir_path=classify_loop_path,
                    )

                    sum_table_path = get_file_path(
                        sum_table_file,
                        dir_str=loop_nuc_name,
                        dir_path=classify_loop_path,
                    )

                    classify_report_table_path = get_file_path(
                        classify_report_table_file,
                        dir_str=loop_nuc_name,
                        dir_path=classify_loop_path,
                    )

                    pred_matrix_path = get_file_path(
                        pred_matrix_file,
                        dir_str=loop_nuc_name,
                        dir_path=classify_loop_path,
                    )

                    nuc_df = mask_equal(df, nuc_class_col, nuc_class)

                    if len(nuc_df) > 0:

                        chi1_resids = None
                        if loop_name == sw2_name:
                            chi1_resids = 71

                        dih_df = build_dih_table(
                            df=nuc_df,
                            dih_dict=dih_dict,
                            bb_resids=loop_resids,
                            chi1_resids=chi1_resids,
                        )

                        cluster_df = load_table(cluster_table_path)

                        build_dih_matrix(
                            fit_df=cluster_df,
                            pred_df=dih_df,
                            max_norm_path=pred_matrix_path,
                        )

                        fit_matrix = load_matrix(fit_matrix_path)
                        pred_matrix = load_matrix(pred_matrix_path)

                        classify_matrix(
                            cluster_df=cluster_df,
                            pred_df=dih_df,
                            fit_matrix=fit_matrix,
                            pred_matrix=pred_matrix,
                            result_table_path=result_table_path,
                            sum_table_path=sum_table_path,
                            report_table_path=classify_report_table_path,
                            max_nn_dist=0.45,
                            only_save_pred=True,
                            reorder_class=False,
                        )

                        result_df = load_table(result_table_path)
                        sum_df = load_table(sum_table_path)
                        classify_report_df = load_table(classify_report_table_path)

                        result_df[loop_col] = loop_name
                        sum_df[loop_col] = loop_name
                        classify_report_df[loop_col] = loop_name

                        result_df[nuc_class_col] = nuc_class
                        sum_df[nuc_class_col] = nuc_class
                        classify_report_df[nuc_class_col] = nuc_class

                        loop_result_df = pd.concat(
                            [loop_result_df, result_df], sort=False
                        )
                        loop_sum_df = pd.concat([loop_sum_df, sum_df], sort=False)
                        loop_classify_report_df = pd.concat(
                            [loop_classify_report_df, classify_report_df], sort=False
                        )

                loop_result_table_path = get_file_path(
                    result_table_file, dir_str=loop_name, dir_path=out_path
                )
                loop_sum_table_path = get_file_path(
                    sum_table_file, dir_str=loop_name, dir_path=out_path
                )
                loop_classify_report_table_path = get_file_path(
                    classify_report_table_file,
                    dir_str=loop_name,
                    dir_path=out_path,
                )

                save_table(loop_result_table_path, loop_result_df)
                save_table(loop_sum_table_path, loop_sum_df)
                save_table(loop_classify_report_table_path, loop_classify_report_df)

                loop_result_df = loop_result_df.rename(columns={cluster_col: loop_name})

                loop_result_df = loop_result_df.loc[
                    :, [core_path_col, modelid_col, chainid_col, loop_name]
                ]

                df = merge_tables(df, loop_result_df)

            if sw1_gtp_name in lst_col(df, sw1_name, unique=True):
                dist_df = build_dist_table(
                    mask_equal(df, sw1_name, sw1_gtp_name),
                    x_resids=[32],
                    y_resids=[bio_lig_col],
                    x_atomids=["OH"],
                    y_atomids=[gtp_atomids],
                    hb_status_col_lst=[hb_status_col],
                    check_hb=True,
                )

                dist_df[hb_status_col] = dist_df[hb_status_col].map(sw1_gtp_dict)

                dist_df = dist_df.loc[
                    :, [core_path_col, modelid_col, chainid_col, hb_status_col]
                ]

                df = merge_tables(df, dist_df)

                df[sw1_name].replace(
                    {
                        sw1_gtp_name: "",
                        sw1_gtp_wat_name: "",
                        sw1_gtp_dir_name: "",
                        sw1_gtp_no_name: "",
                    },
                    inplace=True,
                )
                df[sw1_name] += df[hb_status_col].fillna("").map(str)

                del df[hb_status_col]

            result_table_path = get_file_path(result_table_file, dir_path=out_path)

            save_table(result_table_path, df)

            for loop_name, loop_resids in loop_resid_dict.items():

                pymol_pml_path = get_file_path(
                    f"{loop_name}_{pymol_pml_file}", dir_path=out_path
                )

                if loop_name == sw1_name:
                    stick_resids = [32]
                elif loop_name == sw2_name:
                    stick_resids = [71]

                write_pymol_script(
                    df,
                    pymol_pml_path,
                    group_col=loop_name,
                    stick_resids=stick_resids,
                    loop_resids=loop_resids,
                    style_ribbon=True,
                    thick_bb=False,
                    show_bio=True,
                    color_palette=conf_color_dict[loop_name],
                    sup_group=True,
                    sup_resids=sup_resids,
                    show_resids="1-166",
                )

    print("Rascore classification complete!")


# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import pandas as pd
from tqdm import tqdm

from ..functions import *


def get_index_dist(
    df,
    index,
    x_resids,
    y_resids,
    x_atomids=None,
    y_atomids=None,
    shared_resids=None,
    shared_atomids=None,
    atom_dist_col_lst=None,
    vect_1_col_lst=None,
    vect_2_col_lst=None,
    hb_status_col_lst=None,
    hb_angle_1_col_lst=None,
    hb_angle_2_col_lst=None,
    wmhb_angle_col_lst=None,
    outlier_col_lst=None,
    check_hb=False,
    use_h=False,
    hb_sc=True,
    hb_bb=True,
    min_hb_dist=2.0,
    max_hb_dist=3.2,
    min_wmhb_dist=2.0,
    max_wmhb_dist=3.0,
    min_hb_angle=90,
    max_hb_angle=180,
    min_wmhb_angle=80,
    max_wmhb_angle=140,
    coord_path_col=None,
):

    if coord_path_col is None:
        coord_path_col = core_path_col

    index_df = get_df_at_index(df, index)

    coord_path = index_df.at[index, coord_path_col]
    modelid = index_df.at[index, modelid_col]
    chainid = index_df.at[index, chainid_col]

    val_lst = [x_resids, y_resids]

    if x_atomids is not None:
        val_lst.append(x_atomids)
    if y_atomids is not None:
        val_lst.append(y_atomids)

    if shared_resids is not None:
        val_lst.append(shared_resids)
    if shared_atomids is not None:
        val_lst.append(shared_atomids)

    for i, val in enumerate(val_lst):
        val_lst[i] = type_lst(val)

    x_lst = lst_nums(0, len(val_lst[0]) - 1)

    if use_h:
        coord_path = modify_coord_path(coord_path, return_pdb=True, add_h=True)

    structure = load_coord(coord_path)

    for x in x_lst:

        resid_1 = val_lst[0][x]
        resid_2 = val_lst[1][x]

        if x_atomids is not None:
            atomid_1 = val_lst[2][x]
        else:
            atomid_1 = None

        if y_atomids is not None:
            atomid_2 = val_lst[3][x]
        else:
            atomid_2 = None

        if atom_dist_col_lst is None:
            cont_atom_dist_col = get_dist_col(
                resid_1,
                resid_2,
                x_atomid=lst_to_str(atomid_1, join_txt="+"),
                y_atomid=lst_to_str(atomid_2, join_txt="+"),
            )
        else:
            cont_atom_dist_col = atom_dist_col_lst[x]

        if vect_1_col_lst is None:
            cont_vect_1_col = get_dist_col(
                resid_1,
                resid_2,
                x_atomid=lst_to_str(atomid_1, join_txt="+"),
                y_atomid=lst_to_str(atomid_2, join_txt="+"),
                ext=vect_1_col,
            )
        else:
            cont_vect_1_col = vect_1_col_lst[x]

        if vect_2_col_lst is None:
            cont_vect_2_col = get_dist_col(
                resid_1,
                resid_2,
                x_atomid=lst_to_str(atomid_1, join_txt="+"),
                y_atomid=lst_to_str(atomid_2, join_txt="+"),
                ext=vect_2_col,
            )
        else:
            cont_vect_2_col = vect_2_col_lst[x]

        if hb_status_col_lst is None:
            cont_hb_status_col = get_dist_col(
                resid_1,
                resid_2,
                x_atomid=lst_to_str(atomid_1, join_txt="+"),
                y_atomid=lst_to_str(atomid_2, join_txt="+"),
                ext=hb_status_col,
            )
        else:
            cont_hb_status_col = hb_status_col_lst[x]

        if hb_angle_1_col_lst is None:
            cont_hb_angle_1_col = get_dist_col(
                resid_1,
                resid_2,
                x_atomid=lst_to_str(atomid_1, join_txt="+"),
                y_atomid=lst_to_str(atomid_2, join_txt="+"),
                ext=hb_angle_1_col,
            )
        else:
            cont_hb_angle_1_col = hb_angle_1_col_lst[x]

        if hb_angle_2_col_lst is None:
            cont_hb_angle_2_col = get_dist_col(
                resid_1,
                resid_2,
                x_atomid=lst_to_str(atomid_1, join_txt="+"),
                y_atomid=lst_to_str(atomid_2, join_txt="+"),
                ext=hb_angle_2_col,
            )
        else:
            cont_hb_angle_2_col = hb_angle_2_col_lst[x]

        if wmhb_angle_col_lst is None:
            cont_wmhb_angle_col = get_dist_col(
                resid_1,
                resid_2,
                x_atomid=lst_to_str(atomid_1, join_txt="+"),
                y_atomid=lst_to_str(atomid_2, join_txt="+"),
                ext=wmhb_angle_col,
            )
        else:
            cont_wmhb_angle_col = wmhb_angle_col_lst[x]

        if outlier_col_lst is None:
            cont_outlier_col = get_dist_col(
                resid_1,
                resid_2,
                x_atomid=lst_to_str(atomid_1, join_txt="+"),
                y_atomid=lst_to_str(atomid_2, join_txt="+"),
                ext=outlier_col,
            )
        else:
            cont_outlier_col = outlier_col_lst[x]

        resid_lst = [resid_1, resid_2]

        for i, resid in enumerate(resid_lst):
            if type(resid) == str:
                resid = df.at[index, resid]
            resid_lst[i] = resid

        result = calc_atom_dist(
            structure=structure,
            chainid_1=chainid,
            modelid_1=modelid,
            resid_1=resid_lst[0],
            atomid_1=atomid_1,
            chainid_2=chainid,
            modelid_2=modelid,
            resid_2=resid_lst[1],
            atomid_2=atomid_2,
            check_hb=check_hb,
            use_h=use_h,
            hb_sc=hb_sc,
            hb_bb=hb_bb,
            min_hb_dist=min_hb_dist,
            max_hb_dist=max_hb_dist,
            min_wmhb_dist=min_wmhb_dist,
            max_wmhb_dist=max_wmhb_dist,
            min_hb_angle=min_hb_angle,
            max_hb_angle=max_hb_angle,
            min_wmhb_angle=min_wmhb_angle,
            max_wmhb_angle=max_wmhb_angle,
            return_vect=True,
        )

        atom_dist = result[0]
        vect_1 = result[1]
        vect_2 = result[2]

        if check_hb:
            hb_status = result[3]
            hb_angle_1 = result[4]
            hb_angle_2 = result[5]
            wmhb_angle = result[6]
            outlier_status = result[7]

            if hb_status == wmhb_name:
                if shared_resids is not None:
                    shared_resid = val_lst[4][x]

                    if shared_atomids is not None:
                        shared_atomid = val_lst[5][x]
                    else:
                        shared_atomid = None

                    shared_result_1 = calc_atom_dist(
                        structure=structure,
                        chainid_1=chainid,
                        modelid_1=modelid,
                        resid_1=resid_lst[0],
                        atomid_1=atomid_1,
                        chainid_2=chainid,
                        modelid_2=modelid,
                        resid_2=shared_resid,
                        atomid_2=shared_atomid,
                        check_hb=check_hb,
                        use_h=use_h,
                        hb_sc=hb_sc,
                        hb_bb=hb_bb,
                        min_hb_dist=min_hb_dist,
                        max_hb_dist=max_hb_dist,
                        min_wmhb_dist=min_wmhb_dist,
                        max_wmhb_dist=max_wmhb_dist,
                        min_hb_angle=min_hb_angle,
                        max_hb_angle=max_hb_angle,
                        min_wmhb_angle=min_wmhb_angle,
                        max_wmhb_angle=max_wmhb_angle,
                        return_vect=True,
                    )
                    shared_result_2 = calc_atom_dist(
                        structure=structure,
                        chainid_1=chainid,
                        modelid_1=modelid,
                        resid_1=shared_resid,
                        atomid_1=shared_atomid,
                        chainid_2=chainid,
                        modelid_2=modelid,
                        resid_2=resid_lst[1],
                        atomid_2=atomid_2,
                        check_hb=check_hb,
                        use_h=use_h,
                        hb_sc=hb_sc,
                        hb_bb=hb_bb,
                        min_hb_dist=min_hb_dist,
                        max_hb_dist=max_hb_dist,
                        min_wmhb_dist=min_wmhb_dist,
                        max_wmhb_dist=max_wmhb_dist,
                        min_hb_angle=min_hb_angle,
                        max_hb_angle=max_hb_angle,
                        min_wmhb_angle=min_wmhb_angle,
                        max_wmhb_angle=max_wmhb_angle,
                        return_vect=True,
                    )

                    if (shared_result_1[3] == no_hb_name) or (
                        shared_result_2[3] == no_hb_name
                    ):
                        hb_status = no_hb_name
                        outlier_status = True

        index_df.at[
            index,
            cont_atom_dist_col,
        ] = atom_dist

        index_df.at[
            index,
            cont_vect_1_col,
        ] = vect_1

        index_df.at[
            index,
            cont_vect_2_col,
        ] = vect_2

        if check_hb:
            index_df.at[
                index,
                cont_hb_status_col,
            ] = hb_status

            index_df.at[
                index,
                cont_hb_angle_1_col,
            ] = hb_angle_1

            index_df.at[
                index,
                cont_hb_angle_2_col,
            ] = hb_angle_2

            index_df.at[
                index,
                cont_wmhb_angle_col,
            ] = wmhb_angle
            index_df.at[
                index,
                cont_outlier_col,
            ] = outlier_status

    return index_df


def build_dist_table(
    df,
    x_resids,
    y_resids,
    x_atomids=None,
    y_atomids=None,
    shared_resids=None,
    shared_atomids=None,
    dist_table_path=None,
    atom_dist_col_lst=None,
    hb_status_col_lst=None,
    hb_angle_1_col_lst=None,
    hb_angle_2_col_lst=None,
    wmhb_angle_col_lst=None,
    outlier_col_lst=None,
    check_hb=False,
    use_h=False,
    hb_sc=True,
    hb_bb=True,
    min_hb_dist=2.0,
    max_hb_dist=3.2,
    min_wmhb_dist=2.0,
    max_wmhb_dist=3.0,
    min_hb_angle=90,
    max_hb_angle=180,
    min_wmhb_angle=80,
    max_wmhb_angle=140,
    coord_path_col=None,
):

    dist_df = pd.DataFrame()

    for index in tqdm(
        list(df.index.values), desc="Building distance table", position=0, leave=True
    ):
        dist_df = pd.concat(
            [
                dist_df,
                get_index_dist(
                    df,
                    index,
                    x_resids,
                    y_resids,
                    x_atomids=x_atomids,
                    y_atomids=y_atomids,
                    shared_resids=shared_resids,
                    shared_atomids=shared_atomids,
                    atom_dist_col_lst=atom_dist_col_lst,
                    hb_status_col_lst=hb_status_col_lst,
                    hb_angle_1_col_lst=hb_angle_1_col_lst,
                    hb_angle_2_col_lst=hb_angle_2_col_lst,
                    wmhb_angle_col_lst=wmhb_angle_col_lst,
                    outlier_col_lst=outlier_col_lst,
                    check_hb=check_hb,
                    use_h=use_h,
                    hb_sc=hb_sc,
                    hb_bb=hb_bb,
                    min_hb_dist=min_hb_dist,
                    max_hb_dist=max_hb_dist,
                    min_wmhb_dist=min_wmhb_dist,
                    max_wmhb_dist=max_wmhb_dist,
                    min_hb_angle=min_hb_angle,
                    max_hb_angle=max_hb_angle,
                    min_wmhb_angle=min_wmhb_angle,
                    max_wmhb_angle=max_wmhb_angle,
                    coord_path_col=coord_path_col,
                ),
            ],
            sort=False,
        )

    dist_df = dist_df.reset_index(drop=True)

    print("Built distance table!")

    if dist_table_path is not None:
        save_table(dist_table_path, dist_df)
    else:
        return dist_df
